@@ -1,6 +1,3 @@
-# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
@@ -66,7 +63,7 @@ config.generate_config()
 
 
 :author:    Logilab
-:copyright: 2003-2006 LOGILAB S.A. (Paris, FRANCE)
+:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
 """
 
@@ -392,6 +389,10 @@ class OptionsManagerMixIn(object):
             self._short_options[opt_dict['short']] = opt_name
             args.append('-' + opt_dict['short'])
             del opt_dict['short']
+        available_keys = set(self._optik_parser.option_class.ATTRS)
+        for key in opt_dict.keys():
+            if not key in available_keys:
+                opt_dict.pop(key)
         return args, opt_dict
             
     def cb_set_provider_option(self, option, opt_name, value, parser):
@@ -479,30 +480,11 @@ class OptionsManagerMixIn(object):
             for section, option, optdict in provider.all_options():
                 if onlysection is not None and section != onlysection:
                     continue
-                default = provider.option_default(option, optdict)
-                if optdict['type'] == 'password':
-                    defaultstr = ': '
-                elif default is REQUIRED:
-                    defaultstr = '(required): '
-                else:
-                    if optdict.get('inputlevel', 0) > inputlevel:
-                        provider.set_option(option, default, opt_dict=optdict)
-                        continue
-                    defaultstr = '(default: %s): ' % format_option_value(optdict, default)
-                print ':%s:' % option
-                print optdict.get('help') or option
-                inputfunc = INPUT_FUNCTIONS[optdict['type']]
-                value = inputfunc(optdict, defaultstr)
-                while default is REQUIRED and not value:
-                    print 'please specify a value'
-                    value = inputfunc(optdict, '%s: ' % option)
-                if value is None and default is not None:
-                    value = default
-                provider.set_option(option, value, opt_dict=optdict)
+                provider.input_option(option, optdict, inputlevel)
         # now we can generate the configuration file
         if stream is not None:
             self.generate_config(stream)
-        
+
     def load_config_file(self):
         """dispatch values previously read from a configuration file to each
         options provider)
@@ -570,7 +552,7 @@ class OptionsManagerMixIn(object):
         return self._optik_parser.format_help()
     
 
-class Method:
+class Method(object):
     """used to ease late binding of default method (so you can define options
     on the class using default methods on the configuration instance)
     """
@@ -588,9 +570,8 @@ class Method:
         return getattr(self._inst, self.method)()
 
         
-class OptionsProviderMixIn:
-    """Mixin to provide options to an OptionsManager
-    """
+class OptionsProviderMixIn(object):
+    """Mixin to provide options to an OptionsManager"""
     
     # those attributes should be overridden
     priority = -1
@@ -615,6 +596,8 @@ class OptionsProviderMixIn:
             if action != 'callback':
                 # callback action have no default
                 default = self.option_default(opt_name, opt_dict)
+                if default is REQUIRED:
+                    continue
                 self.set_option(opt_name, default, action, opt_dict)
 
     def option_default(self, opt_name, opt_dict=None):
@@ -675,6 +658,28 @@ class OptionsProviderMixIn:
                 _list.append(value)
         else:
             raise UnsupportedAction(action)
+
+    def input_option(self, option, optdict, inputlevel=99):
+        default = self.option_default(option, optdict)
+        if default is REQUIRED:
+            defaultstr = '(required): '
+        elif optdict.get('inputlevel', 0) > inputlevel:
+            self.set_option(option, default, opt_dict=optdict)
+            return
+        elif optdict['type'] == 'password' or default is None:
+            defaultstr = ': '
+        else:
+            defaultstr = '(default: %s): ' % format_option_value(optdict, default)
+        print ':%s:' % option
+        print optdict.get('help') or option
+        inputfunc = INPUT_FUNCTIONS[optdict['type']]
+        value = inputfunc(optdict, defaultstr)
+        while default is REQUIRED and not value:
+            print 'please specify a value'
+            value = inputfunc(optdict, '%s: ' % option)
+        if value is None and default is not None:
+            value = default
+        self.set_option(option, value, opt_dict=optdict)
             
     def get_option_def(self, opt_name):
         """return the dictionary defining an option given it's name"""
@@ -778,7 +783,7 @@ class Configuration(ConfigurationMixIn):
         super(Configuration, self).__init__(config_file=config_file, usage=usage, version=version)
 
 
-class OptionsManager2ConfigurationAdapter:
+class OptionsManager2ConfigurationAdapter(object):
     """Adapt an option manager to behave like a
     `logilab.common.configuration.Configuration` instance
     """
@@ -876,9 +881,8 @@ def merge_options(options):
     for i in range(len(options)-1, -1, -1):
         optname, optdict = options[i]
         if optname in alloptions:
-            options.pop(alloptions[optname][1])
-            alloptions[optname][0].update(optdict)
-            alloptions[optname][1] = i
+            options.pop(i)
+            alloptions[optname].update(optdict)
         else:
-            alloptions[optname] = [optdict, i]
+            alloptions[optname] = optdict
     return tuple(options)
