@@ -1,6 +1,3 @@
-# Copyright (c) 2006-2007 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
@@ -13,7 +10,13 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""this module contains some function/method decorators"""
+"""this module contains some function/method decorators
+
+:author:    Logilab
+:copyright: 2006-2008 LOGILAB S.A. (Paris, FRANCE)
+:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
+"""
+__docformat__ = "restructuredtext en"
 
 # XXX rewrite so we can use the decorator syntax when keyarg has to be specified
 
@@ -53,6 +56,7 @@ def cached(callableobj, keyarg=None):
                 _cache[key] = callableobj(self, *args, **kwargs)
             return _cache[key]
         return cache_wrapper2
+
     def cache_wrapper3(self, *args):
         cache = '_%s_cache_' % callableobj.__name__
         #print 'cache3?', cache, self, args
@@ -100,3 +104,21 @@ class wproperty(object):
     def __get__(self, obj, cls):
         assert obj is not None
         return getattr(obj, self.attrname)
+
+
+class classproperty(object):
+    def __init__(self, get):
+        self.get = get
+    def __get__(self, inst, cls):
+        return self.get(cls)
+
+from time import clock
+def timed(f):
+    def wrap(*args, **kwargs):
+        t = clock()
+        #for i in range(100):
+        res = f(*args, **kwargs)
+        print '%s time: %.9f' % (f.__name__, clock() - t)
+        return res
+    return wrap
+
