@@ -28,6 +28,14 @@ import imp
 import __builtin__
 
 
+try:
+    import django
+    from logilab.common.modutils import modpath_from_file, load_module_from_modpath
+    DJANGO_FOUND = True
+except ImportError:
+    DJANGO_FOUND = False
+
+
 ## coverage hacks, do not read this, do not read this, do not read this
 
 # hey, but this is an aspect, right ?!!!
@@ -86,6 +94,7 @@ else:
     unittest.FunctionTestCase.__bases__ = (testlib.TestCase,)
 
 
+
 def this_is_a_testfile(filename):
     """returns True if `filename` seems to be a test file"""
     filename = osp.basename(filename)
@@ -100,7 +109,7 @@ def this_is_a_testdir(dirpath):
     return osp.basename(dirpath) in ('test', 'tests', 'unittests')
 
 
-def autopath(projdir=os.getcwd()):
+def project_root(projdir=os.getcwd()):
     """try to find project's root and add it to sys.path"""
     curdir = osp.abspath(projdir)
     previousdir = curdir
@@ -111,9 +120,6 @@ def autopath(projdir=os.getcwd()):
             break
         previousdir = curdir
         curdir = newdir
-    else:
-        sys.path.insert(0, curdir)
-    sys.path.insert(0, '')
     return previousdir
 
 
@@ -142,6 +148,14 @@ class GlobalTestReport(object):
         if not testresult.wasSuccessful():
             problems = len(testresult.failures) + len(testresult.errors)
             self.errmodules.append((filename[:-3], problems, ran))
+
+
+    def failed_to_test_module(self, filename):
+        """called when the test module could not be imported by unittest
+        """
+        self.errors += 1
+        self.errmodules.append((filename[:-3], 1, 1))
+        
     
     def __str__(self):
         """this is just presentation stuff"""
@@ -225,15 +239,14 @@ class PyTester(object):
                 self.testonedir(dirname, exitfirst)
                 dirs[:] = []
 
-
-
+ 
     def testonedir(self, testdir, exitfirst=False):
         """finds each testfile in the `testdir` and runs it"""
         for filename in abspath_listdir(testdir):
             if this_is_a_testfile(filename):
                 # run test and collect information
                 prog = self.testfile(filename, batchmode=True)
-                if exitfirst and not prog.result.wasSuccessful():
+                if exitfirst and (prog is None or not prog.result.wasSuccessful()):
                     break
         # clean local modules
         remove_local_modules_from_sys(testdir)
@@ -249,15 +262,149 @@ class PyTester(object):
         if dirname:
             os.chdir(dirname)
         modname = osp.basename(filename)[:-3]
+        try:
+            print >>sys.stderr, ('  %s  ' % osp.basename(filename)).center(70, '=')
+        except TypeError: # < py 2.4 bw compat
+            print >>sys.stderr, ('  %s  ' % osp.basename(filename)).center(70)
+        try:
+            try:
+                tstart, cstart = time(), clock()
+                testprog = testlib.unittest_main(modname, batchmode=batchmode, cvg=self.cvg)
+                tend, cend = time(), clock()
+                ttime, ctime = (tend - tstart), (cend - cstart)
+                self.report.feed(filename, testprog.result, ttime, ctime)
+                return testprog
+            except (KeyboardInterrupt, SystemExit):
+                raise
+            except Exception, exc:
+                self.report.failed_to_test_module(filename)
+                print 'unhandled exception occured while testing', modname
+                import traceback
+                traceback.print_exc()
+                return None                
+        finally:
+            if dirname:
+                os.chdir(here)
+
+
+
+class DjangoTester(PyTester):
+
+    def __init__(self, cvg):
+        super(DjangoTester, self).__init__(cvg)
+
+
+    def load_django_settings(self, dirname):
+        """try to find project's setting and load it"""
+        curdir = osp.abspath(dirname)
+        previousdir = curdir
+        while not osp.isfile(osp.join(curdir, 'settings.py')) and \
+                  osp.isfile(osp.join(curdir, '__init__.py')):
+            newdir = osp.normpath(osp.join(curdir, os.pardir))
+            if newdir == curdir:
+                raise AssertionError('could not find settings.py')
+            previousdir = curdir
+            curdir = newdir
+        # late django initialization
+        settings = load_module_from_modpath(modpath_from_file(osp.join(curdir, 'settings.py')))
+        from django.core.management import setup_environ
+        setup_environ(settings)
+        settings.DEBUG = False
+        self.settings = settings
+        # add settings dir to pythonpath since it's the project's root
+        if curdir not in sys.path:
+            sys.path.insert(1, curdir)
+
+    def before_testfile(self):
+        # Those imports must be done **after** setup_environ was called
+        from django.test.utils import setup_test_environment
+        from django.test.utils import create_test_db
+        setup_test_environment()
+        create_test_db(verbosity=0)
+        self.dbname = self.settings.TEST_DATABASE_NAME
+        
+
+    def after_testfile(self):
+        # Those imports must be done **after** setup_environ was called
+        from django.test.utils import teardown_test_environment
+        from django.test.utils import destroy_test_db
+        teardown_test_environment()
+        print 'destroying', self.dbname
+        destroy_test_db(self.dbname, verbosity=0)
+        
+
+    def testall(self, exitfirst=False):
+        """walks trhough current working directory, finds something
+        which can be considered as a testdir and runs every test there
+        """
+        for dirname, dirs, files in os.walk(os.getcwd()):
+            for skipped in ('CVS', '.svn', '.hg'):
+                if skipped in dirs:
+                    dirs.remove(skipped)
+            if 'tests.py' in files:
+                self.testonedir(dirname, exitfirst)
+                dirs[:] = []
+            else:
+                basename = osp.basename(dirname)
+                if basename in ('test', 'tests'):
+                    print "going into", dirname
+                    # we found a testdir, let's explore it !
+                    self.testonedir(dirname, exitfirst)
+                    dirs[:] = []
+
+
+    def testonedir(self, testdir, exitfirst=False):
+        """finds each testfile in the `testdir` and runs it"""
+        # special django behaviour : if tests are splited in several files,
+        # remove the main tests.py file and tests each test file separately
+        testfiles = [fpath for fpath in abspath_listdir(testdir)
+                     if this_is_a_testfile(fpath)]
+        if len(testfiles) > 1:
+            try:
+                testfiles.remove(osp.join(testdir, 'tests.py'))
+            except ValueError:
+                pass
+        for filename in testfiles:
+            # run test and collect information
+            prog = self.testfile(filename, batchmode=True)
+            if exitfirst and (prog is None or not prog.result.wasSuccessful()):
+                break
+        # clean local modules
+        remove_local_modules_from_sys(testdir)
+
+
+    def testfile(self, filename, batchmode=False):
+        """runs every test in `filename`
+
+        :param filename: an absolute path pointing to a unittest file
+        """
+        here = os.getcwd()
+        dirname = osp.dirname(filename)
+        if dirname:
+            os.chdir(dirname)
+        self.load_django_settings(dirname)
+        modname = osp.basename(filename)[:-3]
         print >>sys.stderr, ('  %s  ' % osp.basename(filename)).center(70, '=')
         try:
-            tstart, cstart = time(), clock()
-            testprog = testlib.unittest_main(modname, batchmode=batchmode, cvg=self.cvg)
-            tend, cend = time(), clock()
-            ttime, ctime = (tend - tstart), (cend - cstart)
-            self.report.feed(filename, testprog.result, ttime, ctime)
-            return testprog
+            try:
+                tstart, cstart = time(), clock()
+                self.before_testfile()
+                testprog = testlib.unittest_main(modname, batchmode=batchmode, cvg=self.cvg)
+                tend, cend = time(), clock()
+                ttime, ctime = (tend - tstart), (cend - cstart)
+                self.report.feed(filename, testprog.result, ttime, ctime)
+                return testprog
+            except SystemExit:
+                raise
+            except Exception, exc:
+                import traceback
+                traceback.print_exc()
+                self.report.failed_to_test_module(filename)
+                print 'unhandled exception occured while testing', modname
+                print 'error: %s' % exc
+                return None                
         finally:
+            self.after_testfile()
             if dirname:
                 os.chdir(here)
 
@@ -318,6 +465,8 @@ def parseargs():
                       "to skip several patterns, use commas")
     parser.add_option('-q', '--quiet', callback=rebuild_cmdline,
                       action="callback", help="Minimal output")
+    parser.add_option('-P', '--profile', default=None, dest='profile',
+                      help="Profile execution and store data in the given file")
 
     try:
         from logilab.devtools.lib.coverage import Coverage
@@ -327,6 +476,11 @@ def parseargs():
     except ImportError:
         pass
 
+    if DJANGO_FOUND:
+        parser.add_option('-J', '--django', dest='django', default=False,
+                          action="store_true",
+                          help='use pytest for django test cases')
+
     # parse the command line
     options, args = parser.parse_args()
     if options.pdb and getattr(options, 'coverage', False):
@@ -352,27 +506,40 @@ def parseargs():
 
 
 def run():
-    rootdir = autopath()
     options, newargs, explicitfile = parseargs()
     # mock a new command line
     sys.argv[1:] = newargs
     covermode = getattr(options, 'coverage', None)
     cvg = None
+    if not '' in sys.path:
+        sys.path.insert(0, '')    
     if covermode:
+        rootdir = project_root()
         # control_import_coverage(rootdir)
         from logilab.devtools.lib.coverage import Coverage
         cvg = Coverage([rootdir])
         cvg.erase()
         cvg.start()
-    tester = PyTester(cvg)
+    if DJANGO_FOUND and options.django:
+        tester = DjangoTester(cvg)
+    else:
+        tester = PyTester(cvg)
+    if explicitfile:
+        cmd, args = tester.testfile, (explicitfile,)
+    elif options.testdir:
+        cmd, args = tester.testonedir, (options.testdir, options.exitfirst)
+    else:
+        cmd, args = tester.testall, (options.exitfirst,)
     try:
         try:
-            if explicitfile:
-                tester.testfile(explicitfile)
-            elif options.testdir:
-                tester.testonedir(options.testdir, options.exitfirst)
+            if options.profile:
+                import hotshot
+                prof = hotshot.Profile(options.profile)
+                prof.runcall(cmd, *args)
+                prof.close()
+                print 'profile data saved in', options.profile
             else:
-                tester.testall(options.exitfirst)
+                 cmd(*args)           
         except SystemExit:
             raise
         except:
