@@ -22,8 +22,6 @@ If no non-option arguments are present, prefixes used are 'test',
 'regrtest', 'smoketest' and 'unittest'.
 
 """
-from __future__ import nested_scopes
-
 import sys
 import os, os.path as osp
 import re
@@ -53,8 +51,10 @@ except ImportError:
 
 from logilab.common.deprecation import class_renamed, deprecated_function, \
      obsolete
-from logilab.common.compat import set, enumerate
+from logilab.common.compat import set, enumerate, any
 from logilab.common.modutils import load_module_from_name
+from logilab.common.debugger import Debugger
+from logilab.common.decorators import cached
 
 __all__ = ['main', 'unittest_main', 'find_tests', 'run_test', 'spawn']
 
@@ -273,38 +273,11 @@ def _count(n, word):
     
 
 ## PostMortem Debug facilities #####
-from pdb import Pdb
-class Debugger(Pdb):
-    def __init__(self, tcbk):
-        Pdb.__init__(self)
-        self.reset()
-        while tcbk.tb_next is not None:
-            tcbk = tcbk.tb_next
-        self._tcbk = tcbk
-        self._histfile = osp.join(os.environ["HOME"], ".pdbhist")
-        
-    def setup_history_file(self):
-        if readline is not None:
-            try:
-                readline.read_history_file(self._histfile)
-            except IOError:
-                pass
-
-    def start(self):
-        self.interaction(self._tcbk.tb_frame, self._tcbk)
-
-    def setup(self, frame, tcbk):
-        self.setup_history_file()
-        Pdb.setup(self, frame, tcbk)
-
-    def set_quit(self):
-        if readline is not None:
-            readline.write_history_file(self._histfile)
-        Pdb.set_quit(self)
-
-def start_interactive_mode(debuggers, descrs):
+def start_interactive_mode(result):
     """starts an interactive shell so that the user can inspect errors
     """
+    debuggers = result.debuggers
+    descrs = result.error_descrs + result.fail_descrs
     if len(debuggers) == 1:
         # don't ask for test name if there's only one failure
         debuggers[0].start()
@@ -312,7 +285,8 @@ def start_interactive_mode(debuggers, descrs):
         while True:
             testindex = 0
             print "Choose a test to debug:"
-            print "\n".join(['\t%s : %s' % (i, descr) for i, descr in enumerate(descrs)])
+            # order debuggers in the same way than errors were printed
+            print "\n".join(['\t%s : %s' % (i, descr) for i, (_, descr) in enumerate(descrs)])
             print "Type 'exit' (or ^D) to quit"
             print
             try:
@@ -323,7 +297,7 @@ def start_interactive_mode(debuggers, descrs):
                 else:
                     try:
                         testindex = int(todebug)
-                        debugger = debuggers[testindex]
+                        debugger = debuggers[descrs[testindex][0]]
                     except (ValueError, IndexError):
                         print "ERROR: invalid test number %r" % (todebug,)
                     else:
@@ -345,7 +319,8 @@ class SkipAwareTestResult(unittest._TextTestResult):
                                                   descriptions, verbosity)
         self.skipped = []
         self.debuggers = []
-        self.descrs = []
+        self.fail_descrs = []
+        self.error_descrs = []
         self.exitfirst = exitfirst
         self.capture = capture
         self.printonly = printonly
@@ -353,23 +328,13 @@ class SkipAwareTestResult(unittest._TextTestResult):
         self.cvg = cvg
         self.pdbclass = Debugger
 
-##     def startTest(self, test):
-##         "Called when the given test is about to be run"
-##         if self.cvg:
-##             self.cvg.start()
-##         return super(SkipAwareTestResult, self).startTest(test)
-
-##     def stopTest(self, test):
-##         "Called when the given test has been run"
-##         ret = super(SkipAwareTestResult, self).startTest(test)
-##         if self.cvg:
-##             self.cvg.stop()
-##         return ret
-        
-    def _create_pdb(self, test_descr):
+    def descrs_for(self, flavour):
+        return getattr(self, '%s_descrs' % flavour.lower())
+    
+    def _create_pdb(self, test_descr, flavour):
+        self.descrs_for(flavour).append( (len(self.debuggers), test_descr) )
         if self.pdbmode:
             self.debuggers.append(self.pdbclass(sys.exc_info()[2]))
-            self.descrs.append(test_descr)
         
     def addError(self, test, err):
         exc_type, exc, tcbk = err
@@ -378,17 +343,19 @@ class SkipAwareTestResult(unittest._TextTestResult):
         else:
             if self.exitfirst:
                 self.shouldStop = True
+            descr = self.getDescription(test)
             super(SkipAwareTestResult, self).addError(test, err)
-            self._create_pdb(self.getDescription(test))
+            self._create_pdb(descr, 'error')
 
     def addFailure(self, test, err):
         if self.exitfirst:
             self.shouldStop = True
+        descr = self.getDescription(test)
         super(SkipAwareTestResult, self).addFailure(test, err)
-        self._create_pdb(self.getDescription(test))
+        self._create_pdb(descr, 'fail')
 
     def addSkipped(self, test, reason):
-        self.skipped.append((test, reason))
+        self.skipped.append((test, self.getDescription(test), reason))
         if self.showAll:
             self.stream.writeln("SKIPPED")
         elif self.dots:
@@ -399,15 +366,15 @@ class SkipAwareTestResult(unittest._TextTestResult):
         self.printSkippedList()
         
     def printSkippedList(self):
-        for test, err in self.skipped:
+        for test, descr, err in self.skipped:
             self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % ('SKIPPED', self.getDescription(test)))
+            self.stream.writeln("%s: %s" % ('SKIPPED', descr))
             self.stream.writeln("\t%s" % err)
 
     def printErrorList(self, flavour, errors):
-        for test, err in errors:
+        for (_, descr), (test, err) in zip(self.descrs_for(flavour), errors):
             self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
+            self.stream.writeln("%s: %s" % (flavour, descr))
             self.stream.writeln(self.separator2)
             self.stream.writeln("%s" % err)
             try:
@@ -431,11 +398,25 @@ class SkipAwareTestResult(unittest._TextTestResult):
                     self.stream.writeln('no stderr'.center(len(self.separator2)))
 
 
+
+class TestSuite(unittest.TestSuite):
+    def run(self, result, runcondition=None):
+        for test in self._tests:
+            if result.shouldStop:
+                break
+            test(result, runcondition)
+        return result
+    
+    # python2.3 compat
+    def __call__(self, *args, **kwds):
+        return self.run(*args, **kwds)
+
+
 class SkipAwareTextTestRunner(unittest.TextTestRunner):
 
     def __init__(self, stream=sys.stderr, verbosity=1,
                  exitfirst=False, capture=False, printonly=None,
-                 pdbmode=False, cvg=None):
+                 pdbmode=False, cvg=None, test_pattern=None, skipped_patterns=()):
         super(SkipAwareTextTestRunner, self).__init__(stream=stream,
                                                       verbosity=verbosity)
         self.exitfirst = exitfirst
@@ -443,13 +424,73 @@ class SkipAwareTextTestRunner(unittest.TextTestRunner):
         self.printonly = printonly
         self.pdbmode = pdbmode
         self.cvg = cvg
-        
-        
+        self.test_pattern = test_pattern
+        self.skipped_patterns = skipped_patterns
+
+    def _this_is_skipped(self, testedname):
+        return any([(pat in testedname) for pat in self.skipped_patterns])
+
+    def _runcondition(self, test, skipgenerator=True):
+        if isinstance(test, InnerTest):
+            testname = test.name
+        else:
+            if isinstance(test, TestCase):
+                meth = test._get_test_method()
+                func = meth.im_func
+                testname = '%s.%s' % (meth.im_class.__name__, func.__name__)
+            elif isinstance(test, types.FunctionType):
+                func = test
+                testname = func.__name__
+            elif isinstance(test, types.MethodType):
+                func = test.im_func
+                testname = '%s.%s' % (test.im_class.__name__, func.__name__)
+            else:
+                return True # Not sure when this happens
+            if is_generator(func) and skipgenerator:
+                return True # Let inner tests decide at run time
+        # print 'testname', testname, self.test_pattern
+        if self._this_is_skipped(testname):
+            return False # this was explicitly skipped
+        if self.test_pattern is None:
+            return True # no pattern
+        try:
+            classpattern, testpattern = self.test_pattern.split('.')
+            klass, name = testname.split('.')
+            return classpattern in klass and testpattern in name
+        except ValueError:
+            return self.test_pattern in testname
+    
     def _makeResult(self):
         return SkipAwareTestResult(self.stream, self.descriptions, self.verbosity,
                                    self.exitfirst, self.capture, self.printonly,
                                    self.pdbmode, self.cvg)
 
+    def run(self, test):
+        "Run the given test case or test suite."
+        result = self._makeResult()
+        startTime = time.time()
+        test(result, self._runcondition)
+        stopTime = time.time()
+        timeTaken = stopTime - startTime
+        result.printErrors()
+        self.stream.writeln(result.separator2)
+        run = result.testsRun
+        self.stream.writeln("Ran %d test%s in %.3fs" %
+                            (run, run != 1 and "s" or "", timeTaken))
+        self.stream.writeln()
+        if not result.wasSuccessful():
+            self.stream.write("FAILED (")
+            failed, errored = map(len, (result.failures, result.errors))
+            if failed:
+                self.stream.write("failures=%d" % failed)
+            if errored:
+                if failed: self.stream.write(", ")
+                self.stream.write("errors=%d" % errored)
+            self.stream.writeln(")")
+        else:
+            self.stream.writeln("OK")
+        return result
+
 
 class keywords(dict):
     """keyword args (**kwargs) support for generative tests"""
@@ -481,37 +522,32 @@ class NonStrictTestLoader(unittest.TestLoader):
     python test_foo.py test_foo1 will run test_foo1
     python test_foo.py test_bar will run FooTC.test_bar1 and BarTC.test_bar2
     """
+    suiteClass = TestSuite
 
     def __init__(self):
         self.skipped_patterns = []
 
-    
     def loadTestsFromNames(self, names, module=None):
         suites = []
         for name in names:
             suites.extend(self.loadTestsFromName(name, module))
         return self.suiteClass(suites)
 
-
     def _collect_tests(self, module):
         tests = {}
         for obj in vars(module).values():
-            if issubclass(type(obj), (types.ClassType, type)) and \
-                   issubclass(obj, unittest.TestCase):
+            if (issubclass(type(obj), (types.ClassType, type)) and
+                 issubclass(obj, unittest.TestCase)):
                 classname = obj.__name__
-                methodnames = []
                 if self._this_is_skipped(classname):
                     continue
+                methodnames = []
                 # obj is a TestCase class
                 for attrname in dir(obj):
                     if attrname.startswith(self.testMethodPrefix):
                         attr = getattr(obj, attrname)
                         if callable(attr):
-                            for pattern in self.skipped_patterns:
-                                if pattern in attrname:
-                                    break
-                            else:
-                                methodnames.append(attrname)
+                            methodnames.append(attrname)
                 # keep track of class (obj) for convenience
                 tests[classname] = (obj, methodnames)
         return tests
@@ -527,7 +563,6 @@ class NonStrictTestLoader(unittest.TestLoader):
         # _tests explicitly
         return suite._tests
     
-        
     def loadTestsFromName(self, name, module=None):
         parts = name.split('.')
         if module is None or len(parts) > 2:
@@ -550,47 +585,28 @@ class NonStrictTestLoader(unittest.TestLoader):
             else:
                 # case python unittest_foo.py something
                 for klass, methodnames in tests.values():
-                    collected += [klass(methodname) for methodname in methodnames
-                                  if self._test_should_be_collected(methodname, pattern)]
+                    collected += [klass(methodname) for methodname in methodnames]
         elif len(parts) == 2:
             # case "MyClass.test_1"
             classname, pattern = parts
             klass, methodnames = tests.get(classname, (None, []))
             for methodname in methodnames:
-                collected = [klass(methodname) for methodname in methodnames
-                             if self._test_should_be_collected(methodname, pattern)]
+                collected = [klass(methodname) for methodname in methodnames]
         return collected
 
-    def _test_should_be_collected(self, methodname, pattern):
-        """returns True if <methodname> matches <pattern>
-        >>> self._test_should_be_collected('test_foobar', 'foo')
-        True
-        >>> self._test_should_be_collected('testfoobar', 'foo')
-        True
-        >>> self._test_should_be_collected('test_foobar', 'test_foo')
-        True
-        >>> self._test_should_be_collected('test_foobar', 'testfoo')
-        False
-        """
-        return pattern in methodname
-
-
     def _this_is_skipped(self, testedname):
-        for pattern in self.skipped_patterns:
-            if pattern in testedname:
-                return True
-        return False
+        return any([(pat in testedname) for pat in self.skipped_patterns])
 
     def getTestCaseNames(self, testCaseClass):
         """Return a sorted sequence of method names found within testCaseClass
         """
-        if self._this_is_skipped(testCaseClass.__name__):
+        is_skipped = self._this_is_skipped
+        if is_skipped(testCaseClass.__name__):
             return []
         testnames = super(NonStrictTestLoader, self).getTestCaseNames(testCaseClass)
-        return [testname for testname in testnames
-                if not self._this_is_skipped(testname)]
-
+        return [testname for testname in testnames if not is_skipped(testname)]
 
+    
 class SkipAwareTestProgram(unittest.TestProgram):
     # XXX: don't try to stay close to unittest.py, use optparse
     USAGE = """\
@@ -616,21 +632,17 @@ Examples:
     def __init__(self, module='__main__', defaultTest=None, batchmode=False, cvg=None):
         self.batchmode = batchmode
         self.cvg = cvg
-        # if cvg:
-        #     cvg.start()
         super(SkipAwareTestProgram, self).__init__(
             module=module, defaultTest=defaultTest,
             testLoader=NonStrictTestLoader())
-        # if cvg:
-        #     cvg.stop()
-       
     
     def parseArgs(self, argv):
         self.pdbmode = False
         self.exitfirst = False
         self.capture = 0
         self.printonly = None
-        skipped_patterns = []
+        self.skipped_patterns = []
+        self.test_pattern = None
         import getopt
         try:
             options, args = getopt.getopt(argv[1:], 'hHvixqcp:s:',
@@ -653,8 +665,8 @@ Examples:
                 if opt in ('-p', '--printonly'):
                     self.printonly = re.compile(value)
                 if opt in ('-s', '--skip'):
-                    skipped_patterns = [pat.strip() for pat in value.split(',')]
-            self.testLoader.skipped_patterns = skipped_patterns
+                    self.skipped_patterns = [pat.strip() for pat in value.split(',')]
+            self.testLoader.skipped_patterns = self.skipped_patterns
             if self.printonly is not None:
                 self.capture += 1
             if len(args) == 0 and self.defaultTest is None:
@@ -665,6 +677,7 @@ Examples:
                     self.test = self.testLoader.loadTestsFromModule(self.module)
                 return
             if len(args) > 0:
+                self.test_pattern = args[0]
                 self.testNames = args
             else:
                 self.testNames = (self.defaultTest,)
@@ -674,18 +687,32 @@ Examples:
 
 
     def runTests(self):
+        if hasattr(self.module, 'setup_module'):
+            try:
+                self.module.setup_module()
+            except Exception, exc:
+                print 'setup_module error:', exc
+                sys.exit(1)
         self.testRunner = SkipAwareTextTestRunner(verbosity=self.verbosity,
                                                   exitfirst=self.exitfirst,
                                                   capture=self.capture,
                                                   printonly=self.printonly,
                                                   pdbmode=self.pdbmode,
-                                                  cvg=self.cvg)
+                                                  cvg=self.cvg,
+                                                  test_pattern=self.test_pattern,
+                                                  skipped_patterns=self.skipped_patterns)
         result = self.testRunner.run(self.test)
+        if hasattr(self.module, 'teardown_module'):
+            try:
+                self.module.teardown_module()
+            except Exception, exc:
+                print 'teardown_module error:', exc
+                sys.exit(1)
         if os.environ.get('PYDEBUG'):
             warn("PYDEBUG usage is deprecated, use -i / --pdb instead", DeprecationWarning)
             self.pdbmode = True
         if result.debuggers and self.pdbmode:
-            start_interactive_mode(result.debuggers, result.descrs)
+            start_interactive_mode(result)
         if not self.batchmode:
             sys.exit(not result.wasSuccessful())
         self.result = result
@@ -814,6 +841,24 @@ def parse_generative_args(params):
 
     return args, kwargs
 
+class InnerTest(tuple):
+    def __new__(cls, name, *data):
+        instance = tuple.__new__(cls, data)
+        instance.name = name
+        return instance
+
+class ClassGetProperty(object):
+    """this is a simple property-like class but for
+    class attributes.
+    """
+    
+    def __init__(self, getter):
+        self.getter = getter
+
+    def __get__(self, obj, objtype):
+        return self.getter(objtype)
+
+
 class TestCase(unittest.TestCase):
     """unittest.TestCase with some additional methods"""
 
@@ -835,6 +880,20 @@ class TestCase(unittest.TestCase):
         self._err = []
         self._current_test_descr = None
 
+    def datadir(cls):
+        """helper attribute holding the standard test's data directory
+        
+        NOTE: this is a logilab's standard
+        """
+        mod = __import__(cls.__module__)
+        return osp.join(osp.dirname(osp.abspath(mod.__file__)), 'data')
+    # cache it (use a class method to cache on class since TestCase is
+    # instantiated for each test run)
+    datadir = ClassGetProperty(cached(datadir))
+
+    def datapath(self, fname):
+        """joins the object's datadir and `fname`"""
+        return osp.join(self.datadir, fname)
 
     def set_description(self, descr):
         """sets the current test's description.
@@ -891,7 +950,7 @@ class TestCase(unittest.TestCase):
         self._start_capture()
         try:
             func(*args, **kwargs)
-        except KeyboardInterrupt:
+        except (KeyboardInterrupt, SystemExit):
             self._stop_capture()
             raise
         except:
@@ -901,7 +960,10 @@ class TestCase(unittest.TestCase):
         self._stop_capture()
         return True
 
-    def __call__(self, result=None):
+    def _get_test_method(self):
+        return getattr(self, self.__testMethodName)
+
+    def __call__(self, result=None, runcondition=None):
         """rewrite TestCase.__call__ to support generative tests
         This is mostly a copy/paste from unittest.py (i.e same
         variable names, same logic, except for the generative tests part)
@@ -915,14 +977,16 @@ class TestCase(unittest.TestCase):
         self._printonly = getattr(result, 'printonly', None)
         # if result.cvg:
         #     result.cvg.start()
+        testMethod = self._get_test_method()
+        if runcondition and not runcondition(testMethod):
+            return # test is skipped
         result.startTest(self)
-        testMethod = getattr(self, self.__testMethodName)
         try:
             if not self.quiet_run(result, self.setUp):
                 return
             # generative tests
             if is_generator(testMethod.im_func):
-                success = self._proceed_generative(result, testMethod)
+                success = self._proceed_generative(result, testMethod, runcondition)
             else:
                 status = self._proceed(result, testMethod)
                 success = (status == 0)
@@ -937,12 +1001,16 @@ class TestCase(unittest.TestCase):
 
 
             
-    def _proceed_generative(self, result, testfunc, args=()):
+    def _proceed_generative(self, result, testfunc, runcondition=None):
         # cancel startTest()'s increment
         result.testsRun -= 1
         self._start_capture()
+        success = True
         try:
             for params in testfunc():
+                if runcondition and not runcondition(testfunc, skipgenerator=False):
+                    if not (isinstance(params, InnerTest) and runcondition(params)):
+                        continue
                 if not isinstance(params, (tuple, list)):
                     params = (params,)
                 func = params[0]
@@ -1112,19 +1180,49 @@ class TestCase(unittest.TestCase):
         """compares two files using difflib"""
         self.assertStreamEqual(file(fname1), file(fname2), junk)
             
-    def assertIsInstance(self, obj, klass, msg=None):
+    def assertIsInstance(self, obj, klass, msg=None, strict=False):
         """compares two files using difflib"""
         if msg is None:
-            msg = '%s is not an instance of %s' % (obj, klass)
-        self.assert_(isinstance(obj, klass), msg)
+            if strict:
+                msg = '%s is not of class %s but of %s'
+            else:
+                msg = '%s is not an instance of %s but of %s'
+            msg = msg % (obj, klass, type(obj))
+        if strict:
+            self.assert_(obj.__class__ is klass, msg)
+        else:
+            self.assert_(isinstance(obj, klass), msg)
 
 
+    def failUnlessRaises(self, excClass, callableObj, *args, **kwargs):
+        """override default failUnlessRaise method to return the raised
+        exception instance.
+        
+        Fail unless an exception of class excClass is thrown
+        by callableObj when invoked with arguments args and keyword
+        arguments kwargs. If a different type of exception is
+        thrown, it will not be caught, and the test case will be
+        deemed to have suffered an error, exactly as for an
+        unexpected exception.
+        """
+        try:
+            callableObj(*args, **kwargs)
+        except excClass, exc:
+            return exc
+        else:
+            if hasattr(excClass, '__name__'):
+                excName = excClass.__name__
+            else:
+                excName = str(excClass)
+            raise self.failureException, "%s not raised" % excName
+
+    assertRaises = failUnlessRaises
+
 import doctest
 
 class SkippedSuite(unittest.TestSuite):
     def test(self):
         """just there to trigger test execution"""
-        print 'goiooo'
         self.skipped_test('doctest module has no DocTestSuite class')
 
 
@@ -1173,7 +1271,7 @@ class DocTest(TestCase):
     without this hack
     """
     skipped = ()
-    def __call__(self, result=None):
+    def __call__(self, result=None, runcondition=None):
         try:
             finder = DocTestFinder(skipped=self.skipped)
             if sys.version_info >= (2, 4):
