@@ -34,7 +34,7 @@ EX_NODE_NOT_FOUND = "No such node as '%s'"
 
 # Base node ###################################################################
 
-class Node :
+class Node(object):
     """a basic tree node, caracterised by an id"""
 
     def __init__(self, nid=None) :
@@ -322,7 +322,7 @@ def post_order_list(node, filter_func=no_filter):
     return l
 
 def pre_order_list(node, filter_func=no_filter):
-    """ 
+    """
     create a list with tree nodes for which the <filter> function returned true
     in a pre order fashion
     """
